/**
 * Copyright (c) 2016 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 *
 * @defgroup bootloader_secure_ble main.c
 * @{
 * @ingroup dfu_bootloader_api
 * @brief Bootloader project main file for secure DFU.
 *
 */

#include <stdint.h>
#include "boards.h"
#include "nrf_mbr.h"
#include "nrf_bootloader.h"
#include "nrf_bootloader_app_start.h"
#include "nrf_bootloader_dfu_timers.h"
#include "nrf_dfu.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "app_error.h"
#include "app_error_weak.h"
#include "nrf_bootloader_info.h"
#include "nrf_delay.h"
#include "nrf_drv_pwm.h"

#define HEAD_LAMP_CONTROL_PIN 15       //Channel 3  /* Pin assignment names in Schematic. */
#define RIGHT_INDICATOR_CONTROL_PIN 17 //Channel 8
#define LEFT_INDICATOR_CONTROL_PIN 19  //Channel 2
#define HORN_CONTROL_PIN 27            //Channel 1
#define HEAD_LAMP_STATUS_PIN 10       //Status 1
#define RIGHT_INDICATOR_STATUS_PIN 16 //Status 2
#define LEFT_INDICATOR_STATUS_PIN 20 //02  //SW1   **Changed to Int1 pin for helmet detection. Originally pin 20.
#define HORN_STATUS_PIN 14            //IN4
#define WHEEL_LOCK_CONTROL_PIN 11    //IN1
#define EXTERNAL_BUCK_ENABLE_PIN 12  //IN2
#define MOTOR_POWER_ENABLE_PIN 13    //IN3
#define ERROR_INDIDCATOR_LED_PIN 26 
#define LCD_BACKLIGHT_CONTROL_PIN 07
#define HELMET_DETECTION_PIN 20
#define MAX485_CONTROL_PIN   29

static nrf_drv_pwm_t PWM0 = NRF_DRV_PWM_INSTANCE(0);
static nrf_pwm_values_common_t seq_values[] =
{
     0,12500,
};

static nrf_pwm_sequence_t horn_seq =
{
    .values.p_common = seq_values,
    .length          = NRF_PWM_VALUES_LENGTH(seq_values),
    .repeats         = 1,
    .end_delay       = 0
};

static void on_error(void)
{
    NRF_LOG_FINAL_FLUSH();

#if NRF_MODULE_ENABLED(NRF_LOG_BACKEND_RTT)
    // To allow the buffer to be flushed by the host.
    nrf_delay_ms(100);
#endif
#ifdef NRF_DFU_DEBUG_VERSION
    NRF_BREAKPOINT_COND;
#endif
    NVIC_SystemReset();
}


void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    NRF_LOG_ERROR("%s:%d", p_file_name, line_num);
    on_error();
}


void app_error_fault_handler(uint32_t id, uint32_t pc, uint32_t info)
{
    NRF_LOG_ERROR("Received a fault! id: 0x%08x, pc: 0x%08x, info: 0x%08x", id, pc, info);
    on_error();
}


void app_error_handler_bare(uint32_t error_code)
{
    NRF_LOG_ERROR("Received an error: 0x%08x!", error_code);
    on_error();
}

void Peripherals_Init(void) {

 // HEAD_LAMP_STATUS_PIN
  nrf_gpio_cfg_input(HEAD_LAMP_STATUS_PIN, NRF_GPIO_PIN_PULLDOWN);

  // RIGHT_INDICATOR_STATUS_PIN
  nrf_gpio_cfg_input(RIGHT_INDICATOR_STATUS_PIN, NRF_GPIO_PIN_PULLDOWN);

  // LEFT_INDICATOR_STATUS_PIN
  nrf_gpio_cfg_input(LEFT_INDICATOR_STATUS_PIN, NRF_GPIO_PIN_PULLDOWN);

  //HORN_STATUS_PIN
  nrf_gpio_cfg_input(HORN_STATUS_PIN, NRF_GPIO_PIN_PULLDOWN);


  nrf_gpio_cfg_output(HEAD_LAMP_CONTROL_PIN);
  nrf_gpio_cfg_output(RIGHT_INDICATOR_CONTROL_PIN);
  nrf_gpio_cfg_output(LEFT_INDICATOR_CONTROL_PIN);
  nrf_gpio_cfg_output(ERROR_INDIDCATOR_LED_PIN);

  nrf_gpio_cfg_output(EXTERNAL_BUCK_ENABLE_PIN);
  nrf_gpio_cfg_output(LCD_BACKLIGHT_CONTROL_PIN);
  nrf_gpio_cfg_output(MOTOR_POWER_ENABLE_PIN);
  nrf_gpio_cfg_output(WHEEL_LOCK_CONTROL_PIN);
  nrf_gpio_cfg_output(MAX485_CONTROL_PIN);
  nrf_gpio_cfg_output(HORN_CONTROL_PIN);

  nrf_gpio_pin_write(HEAD_LAMP_CONTROL_PIN, false); //false
  nrf_gpio_pin_write(RIGHT_INDICATOR_CONTROL_PIN, false);
  nrf_gpio_pin_write(LEFT_INDICATOR_CONTROL_PIN, false);
  nrf_gpio_pin_write(ERROR_INDIDCATOR_LED_PIN, false);
  nrf_gpio_pin_write(EXTERNAL_BUCK_ENABLE_PIN, false);
  nrf_gpio_pin_write(MOTOR_POWER_ENABLE_PIN, false);
  nrf_gpio_pin_write(LCD_BACKLIGHT_CONTROL_PIN, false);
  nrf_gpio_pin_write(WHEEL_LOCK_CONTROL_PIN, false); //Write 1 to lock
  nrf_gpio_pin_write(MAX485_CONTROL_PIN, false); 
  nrf_gpio_pin_write(HORN_CONTROL_PIN, false);

  nrf_drv_pwm_config_t const config0 =
  {
    .output_pins = 
    {
      LEFT_INDICATOR_CONTROL_PIN,                     // channel 0
      RIGHT_INDICATOR_CONTROL_PIN,             // channel 1
      NRF_DRV_PWM_PIN_NOT_USED,             // channel 2
      NRF_DRV_PWM_PIN_NOT_USED,             // channel 3

    },
    .irq_priority = APP_IRQ_PRIORITY_LOWEST,
    .base_clock   = NRF_PWM_CLK_125kHz,
    .count_mode   = NRF_PWM_MODE_UP,
    .top_value    = 12500,// 6250,   //100ms
    .load_mode    = NRF_PWM_LOAD_COMMON,
    .step_mode    = NRF_PWM_STEP_AUTO//NRF_PWM_STEP_AUTO
  };
  APP_ERROR_CHECK(nrf_drv_pwm_init(&PWM0, &config0, NULL));

}


/**
 * @brief Function notifies certain events in DFU process.
 */
static void dfu_observer(nrf_dfu_evt_type_t evt_type)
{
    switch (evt_type)
    {
        case NRF_DFU_EVT_DFU_FAILED:
        case NRF_DFU_EVT_DFU_ABORTED:
        case NRF_DFU_EVT_DFU_INITIALIZED:
           /* bsp_board_init(BSP_INIT_LEDS);
            bsp_board_led_on(BSP_BOARD_LED_0);
            bsp_board_led_on(BSP_BOARD_LED_1);
            bsp_board_led_off(BSP_BOARD_LED_2);*/
            nrf_gpio_pin_write(RIGHT_INDICATOR_CONTROL_PIN, true);
            nrf_gpio_pin_write(LEFT_INDICATOR_CONTROL_PIN, true);
            break;
        case NRF_DFU_EVT_TRANSPORT_ACTIVATED:
           /* bsp_board_led_off(BSP_BOARD_LED_1);
            bsp_board_led_on(BSP_BOARD_LED_2);*/
            //nrf_gpio_pin_write(RIGHT_INDICATOR_CONTROL_PIN, false);
           // nrf_gpio_pin_write(LEFT_INDICATOR_CONTROL_PIN, false);
            break;
        case NRF_DFU_EVT_DFU_STARTED:
            break;
        default:
            break;
    }
}


/**@brief Function for application main entry. */
int main(void)
{
    uint32_t ret_val;
    Peripherals_Init();

      // removing P21 as reset pin 
    NRF_NVMC->CONFIG = NVMC_CONFIG_WEN_Een << NVMC_CONFIG_WEN_Pos;
    while (NRF_NVMC->READY == NVMC_READY_READY_Busy){};
    NRF_NVMC->ERASEUICR = NVMC_ERASEUICR_ERASEUICR_Erase;
    NRF_UICR->PSELRESET[0] = 0xFFFFFFFF ;
    NRF_UICR->PSELRESET[1] = 0xFFFFFFFF ;

    NRF_NVMC->CONFIG = NVMC_CONFIG_WEN_Ren << NVMC_CONFIG_WEN_Pos;
    while (NRF_NVMC->READY == NVMC_READY_READY_Busy){}

    // Protect MBR and bootloader code from being overwritten.
    ret_val = nrf_bootloader_flash_protect(0, MBR_SIZE, false);
    APP_ERROR_CHECK(ret_val);
    ret_val = nrf_bootloader_flash_protect(BOOTLOADER_START_ADDR, BOOTLOADER_SIZE, false);
    APP_ERROR_CHECK(ret_val);

    (void) NRF_LOG_INIT(nrf_bootloader_dfu_timer_counter_get);
    NRF_LOG_DEFAULT_BACKENDS_INIT();

    NRF_LOG_INFO("Inside main");

    ret_val = nrf_bootloader_init(dfu_observer);
    APP_ERROR_CHECK(ret_val);

    // Either there was no DFU functionality enabled in this project o  r the DFU module detected
    // no ongoing DFU operation and found a valid main application.
    // Boot the main application.
    nrf_bootloader_app_start();

    // Should never be reached.
    NRF_LOG_INFO("After main");
}

/**
 * @}
 */
